/* global BdfApi */

/**
 * Objet de configuration de l'accès à une fichothèque
 * L'objet options permet de conserver des options avec l'objet
 * 
 * @constructor
 * @param {string} fichothequeUrl URL de la fichothèque (doit être fini par un /)
 * @param {Object} options Options conservées par la configuration
  */
BdfApi.Config = function (fichothequeUrl, options) {
    this.fichothequeUrl = fichothequeUrl;
    this.publicApi = false;
    this.version = "action";
    this.apiName = "",
    this.options = {};
    if (options) {
        for(var key in options) {
            let value = options[key];
            switch(key) {
                case "api":
                    this.version = value.version;
                    this.publicApi = value.public;
                    this.apiName = value.name;
                    break;
                case "version":
                    this.version = value;
                    break;
                case "public":
                    this.publicApi = value;
                    break;
                case "name":
                    this.apiName = value;
                    break;
                default:
                    this.options[key] = value;
            }
        }
    }
};

BdfApi.Config.prototype.isPublicApi = function () {
    return this.publicApi;
};

/**
 * Retourne l'URL correspondant à une action sur une fichothèque
 * 
 * @param {string} actionName Nom de l'action
 * @returns {string} URL de l'action
 */
BdfApi.Config.prototype.getActionUrl = function (actionName) {
    return this.fichothequeUrl + actionName;
};

BdfApi.Config.prototype.getApiUrl = function (name, version, endpoint) {
    return this.fichothequeUrl + "api/" + version + "/" + name + "/" + endpoint;
};


/**
 * Retourne l'URL correspondant à une action sur une fichothèque
 * 
 * @param {string} endPointName Nom du point de terminaison
 * @returns {BdfApi.Config.EndPoint} endPoint Définition du point de terminaison
 */
BdfApi.Config.prototype.getEndPoint = function (endPointName) {
    var config = this;
    switch(this.version) {
        case "action":
            return _getActionEndPoint();
        case "v1":
            return _getV1EndPoint();
        default:
            throw new Exception("Unknown version: " + this.version);
    }
    
    function _getActionEndPoint() {
        switch(endPointName) {
            case "fiche-array":
                return __newActionEndpoint("pioche", "fiche");
            case "motcle-array":
                return __newActionEndpoint("pioche", "motcle");
            case "fiches":
                return __newActionEndpoint("selection", "fiches");
            case "thesaurus":
                return __newActionEndpoint("thesaurus", "thesaurus");
            default :
                throw new Exception("Unknown endpoint: " + endPointName);
        }
        
        function __newActionEndpoint(actionName, jsonName) {
            return new BdfApi.Config.EndPoint(config.getActionUrl(actionName), jsonName, "action");
        }
        
    }
    
    function _getV1EndPoint() {
        switch(endPointName) {
            case "fiche-array":
                return __newV1EndPoint("fiche-array");
            case "motcle-array":
                return __newV1EndPoint("motcle-array");
             case "fiches":
                return __newV1EndPoint("fiches");
            case "thesaurus":
                return __newV1EndPoint("thesaurus");
            default :
                throw new Exception("Unknown endpoint: " + endPointName);
        }
        
        function __newV1EndPoint(name) {
            return new BdfApi.Config.EndPoint(config.getApiUrl(config.apiName, config.version, name), "", "v1");
        }
    }
    
    
    
    
};


/**
 * Retourne l'URL de consultation d'une fiche
 * 
 * @param {string} corpus Nom du corpus
 * @param {number} id Identifiant de la fiche
 * @param {string} [format] Nom du gabarit d'affichage à utiliser
 * @returns {string} URL de la fiche
 */
BdfApi.Config.prototype.getFicheUrl = function (corpus, id, format) {
    var url = this.fichothequeUrl + "fiches/" + corpus + "-" + id;
    if (format) {
        url += "-" + format;
    }
    url = url + ".html";
    return url;
};

BdfApi.Config.EndPoint = function (url, jsonName, version) {
    this.url = url;
    this.jsonName = jsonName;
    this.version = version;
};
