/* global BdfApi,Fql */

/**
 * Objet permettant de stocker des éléments récupérés par l'API Ajax * 
 * 
 * @constructor
  */
 BdfApi.Wrapper = function () {
    this.thesaurusMap = new Map();
    this.fichesMap = new Map();
};


/**
 * Retourne le thésaurus correspondant à la clé en argument
 * 
 * @param {string} key Clé du thésaurus 
 * @returns {BdfApi.Wrapper.Thesaurus} Objet Thesaurus
 */
BdfApi.Wrapper.prototype.getThesaurus = function (key) {
    return this.thesaurusMap.get(key);
};


/**
 * Retourne l'objet Fiches correspondant à la clé en argument
 * 
 * @param {string} key Clé d'un ensemble fiches
 * @returns {BdfApi.Wrapper.Fiches} Objet Fiches
 */
BdfApi.Wrapper.prototype.getFiches = function (key) {
    return this.fichesMap.get(key);
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {string} thesaurusName Nom du thésaurus
 * @param {Object} options Options de l'appel
 * @param {string} options.thesaurusKey Clé du thésaurus à utiliser au lieu du nom du thésaurus
 * @param {string} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @returns {function} Fonction d'appel de l'API
 */
BdfApi.Wrapper.prototype.initThesaurus = function (apiConfig, thesaurusName, options) {
    var propertiesMapping;
    var requestParameters = {};
    var thesaurusMap = this.thesaurusMap;
    var thesaurusKey = thesaurusName;
    var requestParameters = {};
    if (options) {
        if (options.thesaurusKey) {
            thesaurusKey = options.thesaurusKey;
        }
        if (options.propertiesMapping) {
            propertiesMapping = options.propertiesMapping;
        }
        if (options.requestParameters) {
            Object.assign(requestParameters, options.requestParameters);
        }
    }
    return function (callback) {
        BdfApi.Ajax.loadThesaurus(apiConfig, thesaurusName, {
            propertiesMapping: propertiesMapping,
            requestParameters: requestParameters,
            callback: function (data) {
                let wrapper = new BdfApi.Wrapper.Thesaurus(data);
                thesaurusMap.set(thesaurusKey, wrapper);
                if (callback) {
                    callback();
                }
            }
        });
    };
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {string} fichesKey Clé à utiliser pour conserver le résultat
 * @param {Object} options Options de l'appel
 * @param {Object} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @param {Fql.FicheQuery} options.Query Instance de Fql.FicheQuery indiquant les fiches à récupérer
 * @param {Object} options.ql Objet correspondant à un objet Options utilisé en argument du constructeur de Fql.FicheQuery 
 * @returns {function} Fonction d'appel de l'API
 */
BdfApi.Wrapper.prototype.initFiches = function (apiConfig, fichesKey, options) {
    var propertiesMapping;
    var requestParameters = {};
    var fichesMap = this.fichesMap;
    var requestParameters = {};
    var queries = null;
    if (options) {
        if (options.propertiesMapping) {
            propertiesMapping = options.propertiesMapping;
        }
        if (options.queries) {
            queries = options.queries;
        }
        if (options.requestParameters) {
            Object.assign(requestParameters, options.requestParameters);
        }
        if (options.sort) {
            requestParameters.sort = options.sort;
        }
    }
    if (!queries) {
        queries = new Fql.FicheQuery({
            corpus: fichesKey
        });
    }
    return function (callback) {
        BdfApi.Ajax.loadFiches(apiConfig, queries, {
            propertiesMapping: propertiesMapping,
            requestParameters: requestParameters,
            callback: function (data) {
                let wrapper = new BdfApi.Wrapper.Fiches(data);
                fichesMap.set(fichesKey, wrapper);
                if (callback) {
                    callback();
                }
            }
        });
    };
};

BdfApi.Wrapper.Thesaurus = function (apiData) {
    var idMap = new Map();
    var idalphaMap = new Map();
    this.apiData = apiData;
    this.idMap = idMap;
    this.idalphaMap = idalphaMap;
    _populate(apiData.thesaurus.array);


    function _populate(array) {
        for(let motcle of array) {
            idMap.set(motcle.id, motcle);
            if (motcle.idalpha) {
                idalphaMap.set(motcle.idalpha, motcle);
            }
            _populate(motcle.children);
        }
    }
    
};

BdfApi.Wrapper.Thesaurus.prototype.getName = function () {
    return this.apiData.thesaurus.name;
};

BdfApi.Wrapper.Thesaurus.prototype.getFirstLevelArray = function () {
    return this.apiData.thesaurus.array;
};

BdfApi.Wrapper.Thesaurus.prototype.getMotcleById = function (id) {
	return this.idMap.get(id);
};

BdfApi.Wrapper.Thesaurus.prototype.getMotcleByIdalpha = function (idalpha) {
    return this.idalphaMap.get(idalpha);
};

BdfApi.Wrapper.Thesaurus.prototype.complete = function (completeFunction) {
    for(let motcle of this.idMap.values()) {
        completeFunction(motcle);
    }
};

BdfApi.Wrapper.Thesaurus.prototype.toMotcleArray = function () {
    var array = new Array();
    for(let motcle of this.idMap.values()) {
        array.push(motcle);
    }
    return array;
};

BdfApi.Wrapper.Fiches = function (apiData) {
    this.apiData = apiData;
    var codeMap = new Map();
    this.codeMap = codeMap;
    this.firstCorpus = null;
    for(let entry of apiData.fiches.entries) {
        for(let fiche of entry.array) {
            codeMap.set(fiche.corpus + "/" + fiche.id, fiche);
        }
    }
    if (apiData.fiches.entries.length > 0) {
        this.firstCorpus = apiData.fiches.entries[0].name;
    }
};

BdfApi.Wrapper.Fiches.prototype.getFicheCount = function () {
    return this.codeMap.size;
};

BdfApi.Wrapper.Fiches.prototype.getFiche = function (corpus, id) {
    if (this.codeMap.size === 0) {
        return null;
    }
    if (arguments.length === 1) {
        id = corpus;
        corpus = this.firstCorpus;
    }
    return this.codeMap.get(corpus + "/" + id);
};

BdfApi.Wrapper.Fiches.prototype.getCorpusEntry = function (corpus) {
    if (!corpus) {
        corpus = this.firstCorpus;
    }
    for(let entry of this.apiData.fiches.entries) {
        if (entry.name === corpus) {
            return entry;
        }
    }
    return null;
};

BdfApi.Wrapper.Fiches.prototype.getFicheArray = function (corpus) {
    if (this.codeMap.size === 0) {
        return [];
    }
    let entry = this.getCorpusEntry(corpus);
    if (!entry) {
        return [];
    } else {
        return entry.array;
    }
};

BdfApi.Wrapper.Fiches.prototype.complete = function (completeFunction) {
    for(let fiche of this.codeMap.values()) {
        completeFunction(fiche);
    }
};
