/* FichothequeLib_Tools - Copyright (c) 2006-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.dom;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.namespaces.ExtractionSpace;
import net.fichotheque.selection.DocumentCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.tools.extraction.builders.AddendaExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.AlbumExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.CorpusExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.DocumentFilterBuilder;
import net.fichotheque.tools.extraction.builders.ExtractionDefBuilder;
import net.fichotheque.tools.extraction.builders.FicheFilterBuilder;
import net.fichotheque.tools.extraction.builders.FilterUnitListBuilder;
import net.fichotheque.tools.extraction.builders.GroupClauseBuilder;
import net.fichotheque.tools.extraction.builders.IllustrationFilterBuilder;
import net.fichotheque.tools.extraction.builders.MotcleFilterBuilder;
import net.fichotheque.tools.extraction.builders.ThesaurusExtractDefBuilder;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.fichotheque.utils.ExtractionUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.attr.AttributesBuilder;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DocumentFragmentHolder;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDOMReader {

    private final Fichotheque fichotheque;

    public ExtractionDOMReader(Fichotheque fichotheque) {
        this.fichotheque = fichotheque;
    }

    public ExtractionDef readExtraction(Element element, DocumentFragmentHolder includeCatalog) {
        NodeList includeList = element.getElementsByTagName("include");
        int includeCount = includeList.getLength();
        if (includeCount > 0) {
            Element[] includeArray = new Element[includeCount];
            for (int i = 0; i < includeCount; i++) {
                includeArray[i] = (Element) includeList.item(i);
            }
            for (int i = 0; i < includeCount; i++) {
                Element includeElement = includeArray[i];
                if (includeElement.getElementsByTagName("include").getLength() > 0) {
                    //@todo Warning
                    continue;
                }
                String fragmentKey = includeElement.getAttribute("fragment-key");
                if (fragmentKey.length() > 0) {
                    DocumentFragment documentFragment = includeCatalog.getFragment(fragmentKey);
                    if (documentFragment != null) {
                        DocumentFragment importDocumentFragment = (DocumentFragment) includeElement.getOwnerDocument().importNode(documentFragment, true);
                        Node parentNode = includeElement.getParentNode();
                        parentNode.replaceChild(importDocumentFragment, includeElement);
                    } else {
                        //@todo Warning
                    }
                }
            }
        }
        ExtractionDefBuilder extractionDefBuilder = ExtractionDefBuilder.init()
                .setTagNameInfo(ExtractionDOMUtils.getTagNameInfo(element));
        DOMUtils.readChildren(element, new RootConsumer(extractionDefBuilder));
        return extractionDefBuilder.toExtractionDef();
    }

    private AddendaExtractDef readAddendaExtractDef(Element element) {
        String name = ExtractionDOMUtils.getExtractName(element);
        TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
        DocumentFilterBuilder documentFilterBuilder = new DocumentFilterBuilder();
        List<DocumentCondition.Entry> entryList = new ArrayList<DocumentCondition.Entry>();
        DOMUtils.readChildren(element, new AddendaExtractConsumer(entryList, documentFilterBuilder));
        if (entryList.isEmpty()) {
            String addendaAttribute = element.getAttribute("addenda");
            if (addendaAttribute.length() > 0) {
                DocumentQuery attributeQuery = ExtractionDOMUtils.getDocumentQuery(addendaAttribute);
                if (attributeQuery != null) {
                    entryList.add(SelectionUtils.toDocumentConditionEntry(attributeQuery));
                }
            }
        }
        return AddendaExtractDefBuilder.init(documentFilterBuilder.toDocumentFilter())
                .setName(name)
                .setTagNameInfo(tagNameInfo)
                .setEntryList(entryList)
                .toAddendaExtractDef();
    }

    private AlbumExtractDef readAlbumExtractDef(Element element) {
        String name = ExtractionDOMUtils.getExtractName(element);
        TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
        IllustrationFilterBuilder illustrationFilterBuilder = new IllustrationFilterBuilder();
        List<IllustrationCondition.Entry> entryList = new ArrayList<IllustrationCondition.Entry>();
        DOMUtils.readChildren(element, new AlbumExtractConsumer(entryList, illustrationFilterBuilder));
        if (entryList.isEmpty()) {
            String albumAttribute = element.getAttribute("album");
            if (albumAttribute.length() > 0) {
                IllustrationQuery attributeQuery = ExtractionDOMUtils.getIllustrationQuery(albumAttribute);
                if (attributeQuery != null) {
                    entryList.add(SelectionUtils.toIllustrationConditionEntry(attributeQuery));
                }
            }
        }
        return AlbumExtractDefBuilder.init(illustrationFilterBuilder.toIllustrationFilter())
                .setName(name)
                .setTagNameInfo(tagNameInfo)
                .setEntryList(entryList)
                .toAlbumExtractDef();
    }

    private CorpusExtractDef readCorpusExtractDef(Element element) {
        CorpusExtractDefBuilder builder = CorpusExtractDefBuilder.init()
                .setName(ExtractionDOMUtils.getExtractName(element))
                .setTagNameInfo(ExtractionDOMUtils.getTagNameInfo(element));
        String sourceAttribute = element.getAttribute("source");
        if (sourceAttribute.equals("master")) {
            builder.setBoolean(CorpusExtractDef.MASTER_BOOLEAN, true);
        }
        String axisAttribute = element.getAttribute("axis");
        if (axisAttribute.equals("descendant")) {
            builder.setBoolean(CorpusExtractDef.DESCENDANTAXIS_BOOLEAN, true);
        }
        List<FicheCondition.Entry> entryList = new ArrayList<FicheCondition.Entry>();
        CorpusExtractConsumer consumer = new CorpusExtractConsumer(builder, entryList);
        DOMUtils.readChildren(element, consumer);
        if (entryList.isEmpty()) {
            String corpusAttribute = element.getAttribute("corpus");
            if (corpusAttribute.length() > 0) {
                FicheQuery attributeQuery = ExtractionDOMUtils.getFicheQuery(corpusAttribute);
                if (attributeQuery != null) {
                    entryList.add(SelectionUtils.toFicheConditionEntry(attributeQuery));
                }
            }
        }
        builder.setEntryList(entryList);
        if (!consumer.isGroupClauseDone()) {
            String sortAttribute = element.getAttribute("sort");
            if (sortAttribute.length() > 0) {
                builder.setClause(CorpusExtractDef.GROUP_CLAUSE, GroupClauseBuilder.getGroupClause(sortAttribute, true));
            }
        }
        if (!consumer.isFicheFilterDone()) {
            builder.setFicheFilter(ExtractionUtils.TITRE_FICHEFILTER);
        }
        return builder.toCorpusExtractDef();
    }

    private ThesaurusExtractDef readThesaurusExtractDef(Element element) {
        ThesaurusExtractDefBuilder builder = ThesaurusExtractDefBuilder.init()
                .setName(ExtractionDOMUtils.getExtractName(element))
                .setTagNameInfo(ExtractionDOMUtils.getTagNameInfo(element));
        String sourceAttribute = element.getAttribute("source");
        if (sourceAttribute.equals("master")) {
            builder.setBoolean(ThesaurusExtractDef.MASTER_BOOLEAN, true);
        }
        List<MotcleCondition.Entry> entryList = new ArrayList<MotcleCondition.Entry>();
        ThesaurusExtractConsumer thesaurusExtractConsumer = new ThesaurusExtractConsumer(builder, entryList);
        DOMUtils.readChildren(element, thesaurusExtractConsumer);
        if (entryList.isEmpty()) {
            String thesaurusAttribute = element.getAttribute("thesaurus");
            if (thesaurusAttribute.length() > 0) {
                MotcleQuery attributeQuery = ExtractionDOMUtils.getMotcleQuery(thesaurusAttribute);
                if (attributeQuery != null) {
                    entryList.add(SelectionUtils.toMotcleConditionEntry(attributeQuery));
                }
            }
        }
        if (!thesaurusExtractConsumer.isMotcleFilterDone()) {
            builder.setMotcleFilter(ExtractionUtils.NONE_MOTCLEFILTER);
        }
        builder.setEntryList(entryList);
        return builder.toThesaurusExtractDef();
    }


    private FicheFilter readFicheFilter(Element element) {
        FicheFilterBuilder builder = FicheFilterBuilder.init()
                .setTagNameInfo(ExtractionDOMUtils.getTagNameInfo(element));
        readFicheFilter(element, new FilterUnitConsumer(builder.getFilterUnitListBuilder()));
        return builder.toFicheFilter();
    }

    private Set<SubsetKey> readCorpusKeySet(Element element) {
        Set<SubsetKey> corpusKeySet = FichothequeUtils.EMPTY_SUBSETKEYSET;
        String corpusAttribute = element.getAttribute("corpus");
        if (corpusAttribute.length() > 0) {
            corpusKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_CORPUS, corpusAttribute);
        }
        return corpusKeySet;
    }

    private void readMotcleFilter(MotcleFilterBuilder motcleFilterBuilder, Element element) {
        TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
        motcleFilterBuilder.setTagNameInfo(tagNameInfo);
        if (StringUtils.isTrue(element.getAttribute("recursive"))) {
            motcleFilterBuilder.setRecursive();
        }
        DOMUtils.readChildren(element, new MotcleFilterConsumer(motcleFilterBuilder));
    }

    private void readFicheFilter(Element parentElement, FilterUnitConsumer filterUnitConsumer) {
        NodeList nodeList = parentElement.getChildNodes();
        int length = nodeList.getLength();
        for (int i = 0; i < length; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    filterUnitConsumer.accept((Element) node);
                    break;
                case Node.TEXT_NODE:
                case Node.CDATA_SECTION_NODE:
                    String text = ((Text) node).getData();
                    if (text != null) {
                        filterUnitConsumer.parse(text);
                    }
            }
        }
    }

    private static Attributes readAttributes(Element element) {
        AttributesBuilder attributesBuilder = new AttributesBuilder();
        String paramsString = element.getAttribute("params");
        if (paramsString.length() > 0) {
            paramsString = paramsString.replace("libtype", "labeltype");
            AttributeUtils.parseInstruction(attributesBuilder, paramsString, ExtractionSpace.EXTRACTION_NAMESPACE, null);
        }
        DOMUtils.readChildren(element, new AttributeConsumer(attributesBuilder));
        return attributesBuilder.toAttributes();
    }


    private class RootConsumer implements Consumer<Element> {

        private final ExtractionDefBuilder extractionDefBuilder;
        private boolean dynamicFicheDone = false;
        private boolean dynamicMotcleDone = false;

        private RootConsumer(ExtractionDefBuilder extractionDefBuilder) {
            this.extractionDefBuilder = extractionDefBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "static":
                    extractionDefBuilder.setStaticTagNameInfo(ExtractionDOMUtils.getTagNameInfo(element));
                    DOMUtils.readChildren(element, new StaticConsumer(extractionDefBuilder));
                    break;
                case "extraitcorpus":
                case "fiches":
                    if (!dynamicFicheDone) {
                        extractionDefBuilder.setDynamicCorpusExtractDef(readCorpusExtractDef(element));
                        dynamicFicheDone = true;
                    }
                    break;
                case "extraitthesaurus":
                case "motcles":
                    if (!dynamicMotcleDone) {
                        extractionDefBuilder.setDynamicThesaurusExtractDef(readThesaurusExtractDef(element));
                        dynamicMotcleDone = true;
                    }
                    break;
                case "fiche":
                    if (!dynamicFicheDone) {
                        FicheFilter ficheFilter = readFicheFilter(element);
                        extractionDefBuilder.setDynamicCorpusExtractDef(CorpusExtractDefBuilder.init(ficheFilter).setTagNameInfo(TagNameInfo.NULL).toCorpusExtractDef());
                        dynamicFicheDone = true;
                    }
                    break;

            }
        }

    }


    private class StaticConsumer implements Consumer<Element> {

        private final ExtractionDefBuilder extractionDefBuilder;

        private StaticConsumer(ExtractionDefBuilder extractionDefBuilder) {
            this.extractionDefBuilder = extractionDefBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "extraitcorpus":
                case "fiches":
                    extractionDefBuilder.addStatic(readCorpusExtractDef(element));
                    break;
                case "extraitthesaurus":
                case "motcles":
                    extractionDefBuilder.addStatic(readThesaurusExtractDef(element));
                    break;
            }
        }

    }


    private class AddendaExtractConsumer implements Consumer<Element> {

        private final DocumentFilterBuilder documentFilterBuilder;
        private final List<DocumentCondition.Entry> entryList;

        private AddendaExtractConsumer(List<DocumentCondition.Entry> entryList, DocumentFilterBuilder documentFilterBuilder) {
            this.entryList = entryList;
            this.documentFilterBuilder = documentFilterBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "document-query":
                case "document-select":
                    entryList.add(SelectionDOMUtils.getDocumentConditionEntry(element));
                    break;
                case "document":
                    TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
                    documentFilterBuilder.setTagNameInfo(tagNameInfo);
                    break;
            }
        }

    }


    private class AlbumExtractConsumer implements Consumer<Element> {

        private final List<IllustrationCondition.Entry> entryList;
        private final IllustrationFilterBuilder illustrationFilterBuilder;

        private AlbumExtractConsumer(List<IllustrationCondition.Entry> entryList, IllustrationFilterBuilder illustrationFilterBuilder) {
            this.entryList = entryList;
            this.illustrationFilterBuilder = illustrationFilterBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "illustration-query":
                case "illustration-select":
                    entryList.add(SelectionDOMUtils.getIllustrationConditionEntry(element));
                    break;
                case "illustration":
                    TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
                    illustrationFilterBuilder.setTagNameInfo(tagNameInfo);
                    break;
            }
        }

    }


    private class FilterUnitConsumer implements Consumer<Element> {

        private final FilterUnitListBuilder builder;

        private FilterUnitConsumer(FilterUnitListBuilder builder) {
            this.builder = builder;
        }

        private void parse(String text) {
            String[] tokens = StringUtils.getTechnicalTokens(text, true);
            for (String token : tokens) {
                parseToken(token);
            }
        }

        private void parseToken(String token) {
            if (token.equals("chrono")) {
                builder.addChronoFilter();
                return;
            }
            if (token.startsWith("phrase_")) {
                String phraseName = token.substring("phrase_".length());
                if (!phraseName.isEmpty()) {
                    builder.addPhraseFilter(phraseName);
                }
            }
            try {
                FieldKey fieldKey = FieldKey.parse(token);
                builder.addFieldKeyFilter(fieldKey, AttributeUtils.EMPTY_ATTRIBUTES);
            } catch (ParseException pe) {
                try {
                    ExtendedIncludeKey extendedIncludeKey = ExtendedIncludeKey.parse(token);
                    builder.addDefaultFilter(extendedIncludeKey, fichotheque, AttributeUtils.EMPTY_ATTRIBUTES);
                } catch (ParseException pe2) {

                }
            }
        }

        @Override
        public void accept(Element element) {
            Attributes attributes = readAttributes(element);
            String tagname = element.getTagName();
            switch (tagname) {
                case "entete":
                    builder.addEnteteFilter();
                    break;
                case "corpsdefiche":
                    builder.addCorpsdeficheFilter();
                    break;
                case "titre":
                    builder.addFieldKeyFilter(FieldKey.TITRE, attributes);
                    break;
                case "soustitre":
                    builder.addFieldKeyFilter(FieldKey.SOUSTITRE, attributes);
                    break;
                case "redacteurs":
                    builder.addFieldKeyFilter(FieldKey.REDACTEURS, attributes);
                    break;
                case "lang":
                    builder.addFieldKeyFilter(FieldKey.LANG, attributes);
                    break;
                case "propriete": {
                    String fieldName = getFieldName(element);
                    if (fieldName.length() > 0) {
                        builder.addFieldNameFilter(FieldKey.PROPRIETE_CATEGORY, fieldName, attributes);
                    }
                    break;
                }
                case "information": {
                    String fieldName = getFieldName(element);
                    if (fieldName.length() > 0) {
                        builder.addFieldNameFilter(FieldKey.INFORMATION_CATEGORY, fieldName, attributes);
                    }
                    break;
                }
                case "section": {
                    String fieldName = getFieldName(element);
                    if (fieldName.length() > 0) {
                        builder.addFieldNameFilter(FieldKey.SECTION_CATEGORY, fieldName, attributes);
                    }
                    break;
                }
                case "lot": {
                    TagNameInfo tagNameInfo = ExtractionDOMUtils.getTagNameInfo(element);
                    FilterUnitListBuilder builder2 = new FilterUnitListBuilder();
                    readFicheFilter(element, new FilterUnitConsumer(builder2));
                    List<FilterUnit> filterUnitList = builder2.toFilterUnitList();
                    if (!filterUnitList.isEmpty()) {
                        builder.addLotFilter(tagNameInfo, filterUnitList);
                    }
                    break;
                }
                case "chrono":
                    builder.addChronoFilter();
                    break;
                case "intitule":
                    builder.addPhraseFilter(FichothequeConstants.FICHE_PHRASE);
                    break;
                case "phrase":
                    String phraseName = element.getAttribute("name");
                    if (!phraseName.isEmpty()) {
                        builder.addPhraseFilter(phraseName);
                    }
                    break;
                case "extraitaddenda":
                case "documents":
                    AddendaExtractDef addendaExtractDef = readAddendaExtractDef(element);
                    builder.addAddendaExtractFilter(addendaExtractDef, attributes);
                    break;
                case "extraitcorpus":
                case "fiches":
                    CorpusExtractDef corpusExtractDef = readCorpusExtractDef(element);
                    builder.addCorpusExtractFilter(corpusExtractDef, attributes);
                    break;
                case "extraitthesaurus":
                case "motcles":
                    ThesaurusExtractDef thesaurusExtractDef = readThesaurusExtractDef(element);
                    builder.addThesaurusExtractFilter(thesaurusExtractDef, attributes);
                    break;
                case "extraitalbum":
                case "illustrations":
                    AlbumExtractDef albumExtractDef = readAlbumExtractDef(element);
                    builder.addAlbumExtractFilter(albumExtractDef, attributes);
                    break;
                case "fiche":
                    builder.addFicheParentageFilter(readFicheFilter(element), attributes, readCorpusKeySet(element));
                    break;
                case "motcle": {
                    MotcleFilterBuilder filterBuilder = new MotcleFilterBuilder(MotcleFilter.DEFAULT_TYPE);
                    readMotcleFilter(filterBuilder, element);
                    builder.addMasterMotcleFilter(filterBuilder.toMotcleFilter(), attributes);
                    break;
                }
            }
        }

        private String getFieldName(Element element) {
            String fieldName = element.getAttribute("name");
            if (fieldName.isEmpty()) {
                fieldName = element.getAttribute("type");
            }
            return fieldName;
        }

    }


    private class CorpusExtractConsumer implements Consumer<Element> {

        private final CorpusExtractDefBuilder builder;
        private final List<FicheCondition.Entry> entryList;
        private boolean groupClauseDone;
        private boolean ficheFilterDone;

        private CorpusExtractConsumer(CorpusExtractDefBuilder builder, List<FicheCondition.Entry> entryList) {
            this.builder = builder;
            this.entryList = entryList;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "fiche-query":
                case "fiche-select":
                    entryList.add(SelectionDOMUtils.getFicheConditionEntry(fichotheque, element));
                    break;
                case "group":
                    builder.setClause(CorpusExtractDef.GROUP_CLAUSE, ExtractionDOMUtils.readGroupClause(element));
                    groupClauseDone = true;
                    break;
                case "title":
                case "intitule":
                    builder.setClause(CorpusExtractDef.TITLE_CLAUSE, ExtractionDOMUtils.readTitleClause(element));
                    break;
                case "pack":
                    builder.setClause(CorpusExtractDef.PACK_CLAUSE, ExtractionDOMUtils.readPackClause(element));
                    break;
                case "fiche":
                    if (!ficheFilterDone) {
                        ficheFilterDone = true;
                        builder.setFicheFilter(readFicheFilter(element));
                    }
                    break;
            }
        }

        public boolean isGroupClauseDone() {
            return groupClauseDone;
        }

        public boolean isFicheFilterDone() {
            return ficheFilterDone;
        }

    }


    private class ThesaurusExtractConsumer implements Consumer<Element> {

        private final ThesaurusExtractDefBuilder builder;
        private final List<MotcleCondition.Entry> entryList;
        private boolean motcleFilterDone;

        private ThesaurusExtractConsumer(ThesaurusExtractDefBuilder builder, List<MotcleCondition.Entry> entryList) {
            this.builder = builder;
            this.entryList = entryList;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "motcle-query":
                case "motcle-select":
                    entryList.add(SelectionDOMUtils.getMotcleConditionEntry(fichotheque, element));
                    break;
                case "motcle":
                    if (!motcleFilterDone) {
                        MotcleFilterBuilder motcleFilterBuilder = new MotcleFilterBuilder(MotcleFilter.DEFAULT_TYPE);
                        readMotcleFilter(motcleFilterBuilder, element);
                        builder.setMotcleFilter(motcleFilterBuilder.toMotcleFilter());
                        motcleFilterDone = true;
                    }
                    break;
                case "title":
                case "intitule":
                    builder.setBoolean(ThesaurusExtractDef.WITHTHESAURUSTITLE_BOOLEAN, true);
                    break;
            }
        }

        public boolean isMotcleFilterDone() {
            return motcleFilterDone;
        }

    }


    private class MotcleFilterConsumer implements Consumer<Element> {

        private final MotcleFilterBuilder motcleFilterBuilder;

        private MotcleFilterConsumer(MotcleFilterBuilder motcleFilterBuilder) {
            this.motcleFilterBuilder = motcleFilterBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagname = element.getTagName();
            switch (tagname) {
                case "labels":
                case "lib":
                    motcleFilterBuilder.setWithLabels(true);
                    break;
                case "level":
                    motcleFilterBuilder.setWithLevel(true);
                    break;
                case "intitule":
                    motcleFilterBuilder.setWithFicheStylePhrase(true);
                    break;
                case "phrase":
                    if (element.getAttribute("name").equals(FichothequeConstants.FICHESTYLE_PHRASE)) {
                        motcleFilterBuilder.setWithFicheStylePhrase(true);
                    }
                    break;
                case "next":
                    MotcleFilterBuilder nextFilterBuilder = new MotcleFilterBuilder(MotcleFilter.NEXT_TYPE);
                    readMotcleFilter(nextFilterBuilder, element);
                    motcleFilterBuilder.setNextFilter(nextFilterBuilder.toMotcleFilter());
                    break;
                case "previous":
                    MotcleFilterBuilder previousFilterBuilder = new MotcleFilterBuilder(MotcleFilter.PREVIOUS_TYPE);
                    readMotcleFilter(previousFilterBuilder, element);
                    motcleFilterBuilder.setPreviousFilter(previousFilterBuilder.toMotcleFilter());
                    break;
                case "motcle":
                    MotcleFilterBuilder childrenFilterBuilder = new MotcleFilterBuilder(MotcleFilter.DEFAULT_TYPE);
                    readMotcleFilter(childrenFilterBuilder, element);
                    motcleFilterBuilder.setChildrenFilter(childrenFilterBuilder.toMotcleFilter());
                    break;
                case "parent":
                    MotcleFilterBuilder parentFilterBuilder = new MotcleFilterBuilder(MotcleFilter.PARENT_TYPE);
                    readMotcleFilter(parentFilterBuilder, element);
                    motcleFilterBuilder.setParentFilter(parentFilterBuilder.toMotcleFilter());
                    break;
                case "fiche":
                    motcleFilterBuilder.getFilterUnitListBuilder().addFicheParentageFilter(readFicheFilter(element), readAttributes(element), readCorpusKeySet(element));
                    break;
                case "extraitcorpus":
                case "fiches":
                    CorpusExtractDef corpusExtractDef = readCorpusExtractDef(element);
                    motcleFilterBuilder.getFilterUnitListBuilder().addCorpusExtractFilter(corpusExtractDef, readAttributes(element));
                    break;
            }
        }


    }


    private static class AttributeConsumer implements Consumer<Element> {

        private final AttributesBuilder attributesBuilder;

        private AttributeConsumer(AttributesBuilder attributeBuilder) {
            this.attributesBuilder = attributeBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "attr":
                    AttributeUtils.readAttrElement(attributesBuilder, element);
                    break;
                case "hide-empty":
                    attributesBuilder.appendValue(ExtractionSpace.HIDE_ATTRIBUTEKEY, ExtractionSpace.EMPTY_HIDE);
                    break;
                case "hide":
                    String hideValue = DOMUtils.readSimpleElement(element);
                    if (!hideValue.isEmpty()) {
                        attributesBuilder.appendValue(ExtractionSpace.HIDE_ATTRIBUTEKEY, hideValue);
                    }
                    break;
                case "groups":
                    String groupsValue = DOMUtils.readSimpleElement(element);
                    if (!groupsValue.isEmpty()) {
                        attributesBuilder.appendValue(ExtractionSpace.GROUPS_ATTRIBUTEKEY, groupsValue);
                    }
                    break;

            }
        }

    }

}
